package uk.ac.man.cs.mig.util.graph.event;

/**
 * User: matthewhorridge<br>
 * The Univeristy Of Manchester<br>
 * Medical Informatics Group<br>
 * Date: Jan 12, 2004<br><br>
 * 
 * matthew.horridge@cs.man.ac.uk<br>
 * www.cs.man.ac.uk/~horridgm<br><br>
 *
 * An interface for listeners intereted in knowing
 * about events generated by a <code>VisualisedObjectManager</code>.
 *
 * @see uk.ac.man.cs.mig.util.graph.controller.VisualisedObjectManager
 */
public interface VisualisedObjectManagerListener
{
	/**
	 * Called when an object is added to the list of objects
	 * being visualised.
	 * @param evt A <code>VisualisedObjectManagerEvent</code> that isShown
	 * information pertaining to the event.
	 */
	public void objectsAdded(VisualisedObjectManagerEvent evt);

	/**
	 * Called when an object that was previously in the list of
	 * objects being visualised was removed.
	 * @param evt A <code>VisualisedObjectManagerEvent</code> that isShown
	 * information pertaining to the event
	 */
	public void objectsRemoved(VisualisedObjectManagerEvent evt);

	/**
	 * Called when the state of an object that is in the list
	 * of objects being visualised changes it's state (which
	 * might affect the visual appearence of the object.
	 * @param evt A <code>VisualisedObjectManagerEvent</code> that isShown
	 * information pertaining to the event
	 */
	public void objectsChanged(VisualisedObjectManagerEvent evt);


	public void parentObjectAdded(VisualisedObjectManagerEvent evt);

	public void parentObjectRemoved(VisualisedObjectManagerEvent evt);

	public void childObjectAdded(VisualisedObjectManagerEvent evt);

	public void childObjectRemoved(VisualisedObjectManagerEvent evt);


}
